// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License.

// WARNING! Do not edit this file manually, your changes will be overwritten.

#pragma once

#ifndef TYPEOBJECT_H
#define TYPEOBJECT_H

#include "IntrospectionSchema.h"

namespace graphql::introspection::object {

class [[nodiscard]] Type final
	: public service::Object
{
private:
	[[nodiscard]] service::AwaitableResolver resolveKind(service::ResolverParams&& params) const;
	[[nodiscard]] service::AwaitableResolver resolveName(service::ResolverParams&& params) const;
	[[nodiscard]] service::AwaitableResolver resolveDescription(service::ResolverParams&& params) const;
	[[nodiscard]] service::AwaitableResolver resolveFields(service::ResolverParams&& params) const;
	[[nodiscard]] service::AwaitableResolver resolveInterfaces(service::ResolverParams&& params) const;
	[[nodiscard]] service::AwaitableResolver resolvePossibleTypes(service::ResolverParams&& params) const;
	[[nodiscard]] service::AwaitableResolver resolveEnumValues(service::ResolverParams&& params) const;
	[[nodiscard]] service::AwaitableResolver resolveInputFields(service::ResolverParams&& params) const;
	[[nodiscard]] service::AwaitableResolver resolveOfType(service::ResolverParams&& params) const;
	[[nodiscard]] service::AwaitableResolver resolveSpecifiedByURL(service::ResolverParams&& params) const;

	[[nodiscard]] service::AwaitableResolver resolve_typename(service::ResolverParams&& params) const;

	struct [[nodiscard]] Concept
	{
		virtual ~Concept() = default;

		[[nodiscard]] virtual service::AwaitableScalar<TypeKind> getKind() const = 0;
		[[nodiscard]] virtual service::AwaitableScalar<std::optional<std::string>> getName() const = 0;
		[[nodiscard]] virtual service::AwaitableScalar<std::optional<std::string>> getDescription() const = 0;
		[[nodiscard]] virtual service::AwaitableObject<std::optional<std::vector<std::shared_ptr<Field>>>> getFields(std::optional<bool>&& includeDeprecatedArg) const = 0;
		[[nodiscard]] virtual service::AwaitableObject<std::optional<std::vector<std::shared_ptr<Type>>>> getInterfaces() const = 0;
		[[nodiscard]] virtual service::AwaitableObject<std::optional<std::vector<std::shared_ptr<Type>>>> getPossibleTypes() const = 0;
		[[nodiscard]] virtual service::AwaitableObject<std::optional<std::vector<std::shared_ptr<EnumValue>>>> getEnumValues(std::optional<bool>&& includeDeprecatedArg) const = 0;
		[[nodiscard]] virtual service::AwaitableObject<std::optional<std::vector<std::shared_ptr<InputValue>>>> getInputFields() const = 0;
		[[nodiscard]] virtual service::AwaitableObject<std::shared_ptr<Type>> getOfType() const = 0;
		[[nodiscard]] virtual service::AwaitableScalar<std::optional<std::string>> getSpecifiedByURL() const = 0;
	};

	template <class T>
	struct [[nodiscard]] Model
		: Concept
	{
		Model(std::shared_ptr<T>&& pimpl) noexcept
			: _pimpl { std::move(pimpl) }
		{
		}

		[[nodiscard]] service::AwaitableScalar<TypeKind> getKind() const final
		{
			return { _pimpl->getKind() };
		}

		[[nodiscard]] service::AwaitableScalar<std::optional<std::string>> getName() const final
		{
			return { _pimpl->getName() };
		}

		[[nodiscard]] service::AwaitableScalar<std::optional<std::string>> getDescription() const final
		{
			return { _pimpl->getDescription() };
		}

		[[nodiscard]] service::AwaitableObject<std::optional<std::vector<std::shared_ptr<Field>>>> getFields(std::optional<bool>&& includeDeprecatedArg) const final
		{
			return { _pimpl->getFields(std::move(includeDeprecatedArg)) };
		}

		[[nodiscard]] service::AwaitableObject<std::optional<std::vector<std::shared_ptr<Type>>>> getInterfaces() const final
		{
			return { _pimpl->getInterfaces() };
		}

		[[nodiscard]] service::AwaitableObject<std::optional<std::vector<std::shared_ptr<Type>>>> getPossibleTypes() const final
		{
			return { _pimpl->getPossibleTypes() };
		}

		[[nodiscard]] service::AwaitableObject<std::optional<std::vector<std::shared_ptr<EnumValue>>>> getEnumValues(std::optional<bool>&& includeDeprecatedArg) const final
		{
			return { _pimpl->getEnumValues(std::move(includeDeprecatedArg)) };
		}

		[[nodiscard]] service::AwaitableObject<std::optional<std::vector<std::shared_ptr<InputValue>>>> getInputFields() const final
		{
			return { _pimpl->getInputFields() };
		}

		[[nodiscard]] service::AwaitableObject<std::shared_ptr<Type>> getOfType() const final
		{
			return { _pimpl->getOfType() };
		}

		[[nodiscard]] service::AwaitableScalar<std::optional<std::string>> getSpecifiedByURL() const final
		{
			return { _pimpl->getSpecifiedByURL() };
		}

	private:
		const std::shared_ptr<T> _pimpl;
	};

	const std::unique_ptr<const Concept> _pimpl;

	[[nodiscard]] service::TypeNames getTypeNames() const noexcept;
	[[nodiscard]] service::ResolverMap getResolvers() const noexcept;

public:
	GRAPHQLSERVICE_EXPORT Type(std::shared_ptr<introspection::Type> pimpl) noexcept;
	GRAPHQLSERVICE_EXPORT ~Type();
};

} // namespace graphql::introspection::object

#endif // TYPEOBJECT_H
